##Sequential conditioning algorithm
###################################################
##input:
#x(covariates),y(outcome), eta(penalty in EBIC: between 0 and 1),
#family (gaussian,binary possion)
#output:
#finalset (the selected final model)
#########################################
SC.glm=function(x,y,eta,family){
  N=nrow(x);p = ncol(x);X=scale(x)
  if (family == 'gaussian'){
    Y = y - mean(y)
  }
  else {
    Y = y
  }
  K=floor(nrow(x)/log(nrow(x)))
  EBIC=numeric(K)
  S = numeric(K) 
  M=apply(X, 2, function(z) deviance(glm(Y~z, family)))
  S[1]=which(M==min(M))
  fit1 <- glm(Y ~X[,S[1]],family)
  offset0= cbind(rep(1,N),X[,S[1]])%*%fit1$coef
  EBIC[1]=deviance(fit1)
  for(k in 1:K){
    M=apply(X, 2, function(z) deviance(glm(Y~z+offset(offset0),family)))
    M[S[1:k]]=1000000
    S[k+1]=which(M==min(M))
    fit <- glm(Y ~X[,S[k+1]]+offset(offset0),family)
    EBIC[k+1]=deviance(fit)+(k)*(log(N)+2*eta*log(p))
    fit1 <- glm(Y ~X[,S[1:(k+1)]],family) 
    offset0=cbind(rep(1,N),X[,S[1:(k+1)]])%*%fit1$coef
    # to confirm five consecutive selections have the increasing EBIC
    k0=5
    if(k>k0){
      if( (EBIC[k-k0]<EBIC[k-(k0-1)]) &
          (sum(EBIC[(k-k0):(k-1)]-EBIC[(k-(k0-1)):(k)] <0)==k0))
        break
    }
    if(k==K-1) {
      break
    }
  }
  bestModel = k-k0
  finalset=S[1:bestModel]
  list(finalset=finalset)
}
#################################################
#Generate simulation data (as in Example 1)
simul_example= function(N, p, family,seed){
  set.seed(seed)
  active=c(1,2,3,4,5,6)
  rho=0.5
  corrmat <- diag(p) 
  corrmat <- rho^abs(row(corrmat)-col(corrmat)) 
  cholmat=chol(corrmat)
  x=matrix(rnorm(N*p,mean=0,sd=1),N,p)
  x=x%*%cholmat
  true_beta  = rep(0,length=p)
  true_beta[1:6]=c(1,-1,1,-1,1,-rho+rho^2-rho^3+rho^4-rho^5)
  if(family=="gaussian"){  
    linpred =  x%*%cbind(true_beta)
    sigma=1
    y = linpred+rnorm(N,mean=0,sd=1)}
  else if(family=="binomial"){
    true_beta  = rep(0,length=p)
    true_beta[1:6]=4*c(1,-1,1,-1,1,-rho+rho^2-rho^3+rho^4-rho^5)
    linpred =  x%*%cbind(true_beta)
    prob = exp(linpred)/(1 + exp(linpred))
    runis = runif(N,0,1)
    y = ifelse(runis < prob,1,0)}
  else if(family=="poisson"){
    true_beta  = rep(0,length=p)
    true_beta[1:6]=0.7*c(1,-1,1,-1,1,-rho+rho^2-rho^3+rho^4-rho^5)
    linpred =  exp(x%*%cbind(true_beta))
    y = rpois(N,lambda=linpred)}
  return(list(x=x,y=y,active=active,beta=true_beta[active],family=family))
}
#########################	
#Working example
dat=simul_example(N=400, p=1000, family="gaussian",seed=12345)
x=dat$x;y=dat$y;N=dim(x)[2];P=dim(x)[1];eta=1-(log(N)/(3*log(p)))
out=SC.glm(x=x,y=y,eta=eta,family="gaussian")
out$finalset
#################
#the end
